/*
 * $Id: canvas.cpp,v 1.4 2001/07/23 06:49:07 inou Exp $
 */

#include "pixmap.h"
#include "setparam.h"
#include "canvas.h"
#include "window.h"

#include <qmessagebox.h>
#include <qaccel.h>
#include <qpainter.h>
#include <qwidget.h>
#include <qevent.h>
#include <qrect.h>
#include <qfiledialog.h>
#include <qmainwindow.h>
#include <qstatusbar.h>

Canvas::Canvas( QWidget *parent, const char *name )
  : QWidget( parent, name, WNorthWestGravity ), // Flag OK?
	pix( width(), height(), -2.5, 2, 1.5, -2, 100, NULL, 0, NULL ),
	maxorbit(10), isOrbit( FALSE ),
	leftButtonPressed( FALSE ), rightButtonPressed( FALSE )
{
  paramnum=0;
  param=NULL;

#ifndef QT_NO_CURSOR
  setCursor( Qt::crossCursor );
#endif

  setMouseTracking( TRUE ) ;
}

void Canvas::setParamNum( int n )
{
  if (paramnum==n)
    return ;

  int m=(paramnum<n) ? paramnum : n;
  paramnum=n;
  double* tmp=param;
  param=new double[n];

  for ( int i=0; i<m; i++ )
    param[i]=tmp[i];

  pix.setParamNum( paramnum );
  pix.setParam( param );
}

void Canvas::resizeEvent( QResizeEvent *e )
{
  QWidget::resizeEvent( e );
  pix.resize( e->size().width(), e->size().height() );
  update();
}

void Canvas::paintEvent( QPaintEvent *e )
{
  QPainter p( this );
  p.drawPixmap( 0, 0, pix );
}

// set plugin and region
// (func, iter, param is already set by MainWindow::loadPlugin)
void Canvas::setPlugin( Plugin *p )
{
  plugin=p;
  pix.setRegion(plugin->TopLeft(),plugin->RightBottom());
}

void Canvas::mouseEvent( QMouseEvent *e )
{
  Point z = getPoint( e );
  if ( leftButtonPressed == TRUE ) {
    emit sendParam( z, paramnum, param );
  }

  if (rightButtonPressed == TRUE ) {
	zoomRect( e );
  }

  if ( isOrbit == TRUE )
    drawOrbit( e );

  ((QMainWindow *)parentWidget())->statusBar()->message(QString("( ") +
			         QString::number(z.x, 'f', 10) + ", " +
				 QString::number(z.y, 'f', 10) + " )" );
}

void Canvas::mousePressEvent( QMouseEvent *e )
{
  if (e->button() == LeftButton)
	leftButtonPressed = TRUE;
  if (e->button() == RightButton) {
	rightButtonPressed = TRUE;
	zoomStart=e->pos();
  }
  mouseEvent( e );
}

void Canvas::mouseReleaseEvent( QMouseEvent *e )
{
  if (e->button() == LeftButton)
	leftButtonPressed = FALSE;
  if (e->button() == RightButton) {
	rightButtonPressed = FALSE;
	zoom(e);
  }
}

void Canvas::mouseMoveEvent( QMouseEvent *e )
{
  mouseEvent( e );
}

Point Canvas::getPoint( QMouseEvent *e )
{
  return pix.getPoint(e->x(), e->y());
}

void Canvas::drawOrbit( QMouseEvent *e )
{
  Point c=getPoint(e);
  Point z=init(c, param);

  repaint(FALSE); // clear screen

  QPainter p( this );

  QPen pen( QColor( 255, 255, 255) );
  p.setPen(pen);
  p.moveTo(pix.getPos(z));

  double r=10*(pix.dWidth()*pix.dWidth() + pix.dHeight()*pix.dHeight()+1);
	
  for (int i=0; i<maxorbit; i++) {
	if (z.abs2() > r) break;
    z = map(z,c,param);
    p.lineTo( pix.getPos(z) );
  }
};

void Canvas::draw()
{
 pix.draw();
 repaint(FALSE);
}

void Canvas::save()
{
  QString fn = QFileDialog::getSaveFileName( QString::null,
					     QString("PNG format (*.png)"),
					     this);
  if ( fn.isEmpty() )
    return; // aborted
  QFileInfo fi( fn );
  if (fi.extension() != "png" )
    fn += ".png";
  pix.save( fn, "PNG" );
}

void Canvas::zoom(QMouseEvent *e)
{
  Point z1 = pix.getPoint( zoomStart.x(), zoomStart.y() );
  Point z2 = getPoint( e );

  MainWindow *win=new MainWindow();
  win->loadPlugin(pluginname);
  win->setRegion(Rect(z1,z2));

  win->setMaxOrbit( maxOrbit() );
  win->setMaxIter( pix.maxIter() );

  double p[paramnum];
  for (int i=0; i<paramnum; i++)
	p[i]=getParam(i);
  win->setParam( p );
  win->setColorMap( pix.colorMap() );
  win->show();

  /*
  pix.setRegion(z1, z2);
  draw();
  */
  update();
}

void Canvas::zoomRect(QMouseEvent *e)
{
  repaint(FALSE); // clear screen

  QPainter p( this );

  QPen pen( QColor( 255, 255, 255) );
  p.setPen(pen);
  p.drawRect( QRect(zoomStart, e->pos()) );
}
