/*
 * $Id: canvas.h,v 1.3 2001/06/15 07:42:54 inou Exp $
 */

#ifndef CANVAS_H
#define CANVAS_H

#include "pixmap.h"
#include "plugin.h"
#include <qwidget.h>
#include <qpoint.h>

class QResizeEvent;
class QAction;

class Point;

class Canvas: public QWidget
{
  Q_OBJECT
public:
  Canvas( QWidget *parent = 0, const char *name = 0 );

  void setParamNum( int n );
  void setParam( double *p )
  { for ( int i=0; i<paramnum; i++ ) param[i]=p[i]; pix.setParam(p); }
  // should use 'emit paramChanged( int );' ?
  void setFunc( Point (*f)(Point, Point, double* ) )
  { map=f; }
  void setInitFunc( Point (*f)(Point, double* ) )
  { init=f; }
  void setIterFunc( int (*f)(Point, double*, int ) )
  { pix.setFunc(f); }
  void loadColorMap( const char *filename ) { pix.loadColorMap(filename); }
  void setColorMap( ColorMap c ) { pix.setColorMap(c); }

  int maxOrbit() { return maxorbit; }
  int paramNum() { return paramnum; }
  ColorMap colorMap() { return pix.colorMap(); }
  double getParam(int i) { if (i<paramnum) return param[i]; else return 0; }
  // checking i<paramnum is unnecessary?

  void draw();
  void setPlugin( Plugin *i );
  void setPluginName( const char *pn ) { pluginname=QString(pn); }
  Rect Region() { return pix.Region(); }

  void mouseEvent( QMouseEvent *e );
  Point getPoint( QMouseEvent *e );
  void drawOrbit( QMouseEvent *e );
  void zoom( QMouseEvent *e );
  void zoomRect( QMouseEvent *e );

public slots:
  void save();
  void setRegion( Rect r ) { pix.setRegion( r ); }
  void setMaxOrbit(int m) { maxorbit=m; }
  void setMaxIter(int m) { pix.setMaxIter(m); }
  void toggleOrbit( bool b ) { isOrbit=b; repaint(FALSE);}
  void mousePressEvent( QMouseEvent *e );
  void mouseReleaseEvent( QMouseEvent *e );
  void mouseMoveEvent( QMouseEvent *e );

signals:
  void sendParam( Point, int, double* );

protected:
  void resizeEvent( QResizeEvent *e );
  void paintEvent( QPaintEvent * );

  Pixmap pix;
  int maxorbit;
  int paramnum;
  double *param;
  Point (*map)(Point, Point, double* ); // calculation function (map once)
  Point (*init)(Point, double* ); // get initial point
  Plugin *plugin;
  QString pluginname;

  bool isOrbit;
  bool leftButtonPressed;
  bool rightButtonPressed;
  QPoint zoomStart;

};

#endif // CANVAS_H
