/*
 * $Id: pixmap.cpp,v 1.2 2001/06/15 07:42:54 inou Exp $
 */

#include "pixmap.h"
#include "colormap.h"

#include <qpixmap.h>
#include <qpainter.h>
#include <qpalette.h>
#include <qpen.h>
#include <qcolor.h>

Pixmap::Pixmap( int w, int h, double xl, double yt,
		  double xr, double yb, int m,
		  int (*f)( Point, double*, int ),
		  int n, double* p )
  : QPixmap( w, h ), colormap()
{
  region.setValue(xl,yt,xr,yb);
  max=m;
  iter=f;
  paramnum=n;
  param=p;

  setDiff();
}

Pixmap::Pixmap( const Pixmap& d )
  : QPixmap( d )
{
  region=d.region;
  iter = d.iter;
  paramnum = d.paramnum;
  param = new double[paramnum];
  for ( int i=0; i<paramnum; i++ )
    param[i]=d.param[i];

  diff=d.diff;
}

void Pixmap::setParamNum( int n )
{
  if (paramnum==n)
    return ;

  int m=(paramnum<n) ? paramnum : n;
  paramnum=n;
  double* tmp=param;
  param=new double[n];

  for ( int i=0; i<m; i++ )
    param[i]=tmp[i];
}

Point Pixmap::getPoint(int ix, int iy)
{
  return Point( region.topLeft()+Point(diff*ix, -diff*iy) );
}


void Pixmap::draw()
{
  fill( black );
  QPainter p( this );
  p.setBrush( blue );

  for (int ix=0; ix<width(); ix++) {
    for (int iy=0; iy<height(); iy++) {
      Point z=getPoint(ix,iy);
      int i=iter( z, param, max );
      if (i>=0) {
	QPen p1( colormap.Color(i%NUM) );
	p.setPen(p1);
	p.drawPoint(ix,iy);
      }
    }
  }

}

void Pixmap::resize( int w, int h )
{
  QPixmap::resize( w, h );
  setDiff();
  draw();
}

QPoint Pixmap::getPos(Point z)
{
  int ix= (int) ((z.x-region.xleft)/diff);
  int iy= (int) ((z.y-region.ytop)/(-diff));
  return QPoint(ix,iy);
}

void Pixmap::setDiff()
{
  double xd = region.width()/width();
  double yd = region.height()/height();
  diff =  (xd > -yd) ? xd : -yd;

}
