/*
 * $Id: pixmap.h,v 1.3 2001/06/15 07:42:54 inou Exp $
 */

#ifndef PIXMAP_H
#define PIXMAP_H

#include "double.h"
#include "colormap.h"

#include <qpixmap.h>
#include <qpoint.h>

class Point;

class Pixmap : public QPixmap
{
public:
  Pixmap( int w, int h, double xl, double yt,
	   double xr, double yb, int m,
	   int (*f)( Point, double*, int ),
	   int n, double* p );
  Pixmap( const Pixmap& d );

  int paramNum() { return paramnum; }
  void setRegion(double xl, double yt, double xr, double yb)
  { region.setValue(xl,yt,xr,yb); setDiff(); }
  void setRegion(Point tl, Point br)
  { region.setValue(tl, br); setDiff(); }
  void setFunc( int (*f)(Point, double*, int ) )
  { iter=f; }
  void setRegion( Rect r ) { region=r; setDiff();}
  void setParamNum( int n );
  void setParam( double* p )
  { for ( int i=0; i<paramnum; i++ ) param[i]=p[i]; }
  void setMaxIter( int m ) { max=m; }
  void loadColorMap( const char *filename ) { colormap.open(filename); }
  void setColorMap( ColorMap c ) { colormap=c; }

  int maxIter() { return max; }
  Rect Region() { return region; }
  ColorMap colorMap() { return colormap; }
  double dWidth() { return region.width(); }
  double dHeight() { return region.height(); }
  
  Point getPoint(int ix, int iy);
  QPoint getPos(Point z);
  void draw(); 
  void resize( int w, int h );

private:
  Rect region;
  double diff;
  int max; // max iteration time
  int paramnum; // number of parameters
  double *param; // parameters for calculation
  int (*iter)(Point, double* , int); // calculation function
  ColorMap colormap;
					  
  void setDiff();

};

#endif // PIXMAP_H
