/*
 * $Id: plugin.h,v 1.2 2001/06/15 07:42:54 inou Exp $
 */

#ifndef PLUGIN_H
#define PLUGIN_H

#include "double.h"

class PluginInfo
{
public:
  PluginInfo(const char *n,
	     double xl, double yt, double xr, double yb,
	     int mi, int mo, int pn, const double *p, const char* c)
  { name=n;  xleft=xl; ytop=yt; xright=xr; ybottom=yb;
  maxiter=mi; maxorbit=mo; paramnum=pn; param=p; childname=c;}
  PluginInfo( const PluginInfo& i )
  { name=i.name;xleft=i.xleft;ytop=i.ytop;xright=i.xright;ybottom=i.ybottom;
  maxiter=i.maxiter;maxorbit=i.maxorbit;paramnum=i.paramnum;param=i.param;
  childname=i.childname;}

  const char *name;

  double xleft, ytop, xright, ybottom;
  int maxiter, maxorbit;
  int paramnum;
  const double *param;

  const char *childname; // the filename of child plugin. NULL if doesn't exist.
};

class Plugin
{
public:
  Plugin(const char *filename);
  ~Plugin();
  Plugin(const Plugin &p);
  void* open(const char *filename);
  int close();

  const char *Name() { return info->name; }
  Point (*Map())( Point, Point, double* ) { return map; }
  Point (*Init())( Point, double* ) { return init; }
  int (*Iter())( Point, double*, int ) { return iter; }

  double XLeft() { return info->xleft; }
  double YTop() { return info->ytop; }
  double XRight() { return info->xright; }
  double YBottom() { return info->ybottom; }
  Point TopLeft() { return Point(info->xleft,info->ytop); }
  Point RightBottom() { return Point(info->xright,info->ybottom); }

  const char *ChildName() { return info->childname; }

  int MaxIter() { return info->maxiter; }
  int MaxOrbit() { return info->maxorbit; }
  int ParamNum() { return info->paramnum; }
  double Param(int i)
  { if (i<info->paramnum) return (info->param)[i]; else return 0; }

private:
  void *handle;
  PluginInfo* info;
  Point (*map)( Point, Point, double* ); // map once
  Point (*init)( Point, double* ); // get initial point
  int (*iter)( Point, double*, int ); // iteration at most (int) times
};

#endif //PLUGIN_H
