/*
 * $Id: setparam.cpp,v 1.3 2001/07/23 06:49:07 inou Exp $
 */

#include "setparam.h"
#include "double.h"

#include <qlabel.h>
#include <qlineedit.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qhbox.h>

SetParam::SetParam( Rect r, int n, double *p, int *mi, int *mo,
		    QWidget *parent=0, const char *name=0, WFlags f=0 )
  : QDialog( parent, name, FALSE, f)
{
  region=r;
  paramnum=n;
  param=p;
  maxiter=mi;
  maxorbit=mo;

  QVBoxLayout* vbox = new QVBoxLayout(this,2);
  
  QGridLayout *grid = new QGridLayout( vbox, 0, 0, 10 );

  QLabel *label;

  // QLineEdit for region
  
  for (int i=0; i<2; i++) {
    for (int j=0; j<2; j++) {
      edregion[i][j]=new QLineEdit(this);
      grid->addWidget( edregion[i][j], i+1, j*2+1 );
    }
  }

  edregion[0][0]->setText( QString::number(region.xleft, 'g', 20) );
  edregion[0][1]->setText( QString::number(region.ytop, 'g', 20) );
  edregion[1][0]->setText( QString::number(region.xright, 'g', 20) );
  edregion[1][1]->setText( QString::number(region.ybottom, 'g', 20) );

  label= new QLabel( edregion[0][0], "XLeft", this );
  grid->addWidget( label, 1, 0 );
  label= new QLabel( edregion[0][1], "YTop", this );
  grid->addWidget( label, 1, 2 );
  label= new QLabel( edregion[1][0], "XRight", this );
  grid->addWidget( label, 2, 0 );
  label= new QLabel( edregion[1][1], "YBottom", this );
  grid->addWidget( label, 2, 2 );

  for (int i=0; i<2; i++) {
    for (int j=0; j<2; j++) {
      edregion[i][j]->setEdited(FALSE);
    }
  }

  // QLineEdit for parameters
  grid->addWidget( new QLabel( QString("Max iteration times"), this) , 3, 2);
  
  // QLineEdit for iteration
  edmaxiter = new QLineEdit( this );
  grid->addWidget(edmaxiter, 4, 3 );
  edmaxiter->setText( QString::number(*maxiter) );
  edmaxiter->setEdited(FALSE);

  label= new QLabel( edmaxiter, "Iteration", this );
  grid->addWidget( label, 4, 2 );

  // QLineEdit for orbit
  edmaxorbit = new QLineEdit( this );
  grid->addWidget(edmaxorbit, 5, 3 );
  edmaxorbit->setText( QString::number(*maxorbit) );
  edmaxorbit->setEdited(FALSE);

  label= new QLabel( edmaxorbit, "Forward orbit", this );
  grid->addWidget( label, 5, 2 );

  edmaxiter->setFocus();

  grid->addWidget( new QLabel( QString("Parameters"), this) , 3, 0);
  
  if (n==0) {
    grid->addWidget( new QLabel( QString("no parameter"), this) , 4, 0);
  } else {
    edparam = new (QLineEdit *)[n];
  
    for (int i=0; i<n; i++) {
      edparam[i] = new QLineEdit( this );
      grid->addWidget(edparam[i], i+4, 1 );
      edparam[i]->setText( QString::number(p[i], 'g', 20) );
      edparam[i]->setEdited(FALSE);      
      QString s;
      s = QString::number(i);
      QLabel *label= new QLabel( edparam[i], s, this );
      grid->addWidget( label, i+4, 0 );
    }
    edparam[0]->setFocus();
  }
  
  grid->setColStretch( 0, 0 );
  grid->setColStretch( 1, 10 );
  grid->setColStretch( 2, 0 );
  grid->setColStretch( 3, 10 );
  
  
  
  QHBoxLayout* hbox = new QHBoxLayout(vbox);
  QPushButton* cancel = new QPushButton("Cancel", this);
  hbox->addWidget(cancel);
  QPushButton* ok = new QPushButton("Ok", this);
  ok->setDefault( TRUE );
  hbox->addWidget(ok);

  setFixedWidth(600);

  connect( ok, SIGNAL(clicked()), this, SLOT(accept()) );
  connect( cancel, SIGNAL(clicked()), this, SLOT(reject()) );

}

void SetParam::accept()
{
  bool b;

  edmaxorbit->text().toInt(&b);
  if (b=FALSE) return;
  *maxorbit = edmaxorbit->text().toInt();
  emit orbitSet();
  
  for (int i=0; i<2; i++) {
    for (int j=0; j<2; j++) {
      edregion[i][j]->text().toDouble(&b);
      if (b=FALSE) return;
    }
  }

  edmaxiter->text().toInt(&b);
  if (b=FALSE) return;

  for (int i=0; i<paramnum; i++ )
    {
      edparam[i]->text().toDouble(&b);
      if (b=FALSE) return;
    }		

  bool edited=FALSE;

  for (int i=0; i<2; i++) {
    for (int j=0; j<2; j++) {
      if (edregion[i][j]->edited()) {
	edited=TRUE;
      }
    }
  }

  for (int i=0; i<paramnum; i++ ) {
    if ( edparam[i]->edited() )
      edited=TRUE;
  }

  if ( edmaxiter->edited() )
    edited=TRUE;

  if (edited) {
    Rect r( edregion[0][0]->text().toDouble(),
	    edregion[0][1]->text().toDouble(),
	    edregion[1][0]->text().toDouble(),
	    edregion[1][1]->text().toDouble() );
    emit setRegion( r );
    
    for (int i=0; i<paramnum; i++ ) {
      param[i]=edparam[i]->text().toDouble();
    }
    
    *maxiter = edmaxiter->text().toInt();
    
    emit paramSet();
  }

  QDialog::accept();
}
