/*
 * $Id: window.cpp,v 1.5 2001/07/23 06:49:07 inou Exp $
 */

#include "window.h"
#include "setparam.h"
#include "canvas.h"
#include "config.h"

#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qkeycode.h>
#include <qstatusbar.h>
#include <qmessagebox.h>
#include <qapplication.h>
#include <qaccel.h>
#include <qwidget.h>
#include <qmenudata.h>
#include <qaction.h>
#include <qfiledialog.h>

MainWindow::MainWindow()
  : QMainWindow( 0, "main window", WDestructiveClose )
{
  canvas = new Canvas( this, "canvas" );
  setCentralWidget( canvas );
  resize( 480, 530 );

  paramnum=0;
  param=NULL;

  child=NULL;

  actionSetParam = new QAction( "Set parameters", "Set &parameters",
				   CTRL+Key_P, this );
  connect( actionSetParam, SIGNAL( activated() ),
	   this, SLOT( setParam() ) );
  //actionSetParam->setEnabled( FALSE );

  actionSetChild =  new QAction( "Create child window", "Create &child window",
				   CTRL+Key_C, this );
  connect( actionSetChild, SIGNAL( activated() ),
	   this, SLOT( createChild() ) );


  QPopupMenu *file = new QPopupMenu( this );
  menuBar()->insertItem( "&File", file );
  QPopupMenu *setting  = new QPopupMenu( this );
  menuBar()->insertItem( "&Settings", setting );

  file->insertItem( "N&ew Window", this,
					SLOT( newWindow() ), ALT+Key_N );
  file->insertItem( "&Save", canvas, SLOT( save() ), ALT+Key_S );
  file->insertSeparator();
  file->insertItem( "C&lose", this, SLOT( close() ), ALT+Key_W );
  file->insertItem( "E&xit", qApp, SLOT( closeAllWindows() ), ALT+Key_Q );

  setting->insertItem( "&Load function",
		       this, SLOT( loadPlugin() ), CTRL+Key_L );
  setting->insertSeparator();
  actionSetParam->addTo( setting );
  actionSetChild->addTo( setting );
  setting->insertItem( "&Revert",
		       this, SLOT( revertPlugin() ), CTRL+Key_R );

  SetOrbitId = setting->insertItem( "&Forward orbit", this,
				    SLOT( toggleOrbit() ), CTRL+Key_F );
  isOrbit=FALSE;

  setting->insertSeparator();
  setting->insertItem( "Load Color &map", this, SLOT( loadColorMap() ) );

  connect( this, SIGNAL(orbitChanged( bool )), canvas, SLOT(toggleOrbit( bool )) );
  statusBar()->message("Ready");

}

void MainWindow::newWindow()
{
  MainWindow *win = new MainWindow();
  win->loadPlugin(DEFAULT_PLUGIN);
  win->show();
}

void MainWindow::setParamNum( int n )
{
  if (paramnum==n)
    return ;

  int m=(paramnum<n) ? paramnum : n;
  paramnum=n;
  double* tmp=param;
  param=new double[n];

  for ( int i=0; i<m; i++ )
    param[i]=tmp[i];

  canvas->setParamNum( paramnum );
  canvas->setParam( param );

  //actionSetParam->setEnabled( n );
}

void MainWindow::setParam()
{
  SetParam *sp= new SetParam( canvas->Region(), paramnum, param, &maxiter, &maxorbit, this, "set param");

  connect( sp, SIGNAL( orbitSet() ), this, SLOT( orbitChanged() ) );
  connect( sp, SIGNAL( paramSet() ), this, SLOT( paramChanged() ) );
  connect( sp, SIGNAL( setRegion( Rect ) ),
		     canvas, SLOT( setRegion( Rect ) ) );

  sp->show();
}

void MainWindow::createChild()
{
  if ((plugin->ChildName() == NULL) || (*(plugin->ChildName()) == '\0')) return;
  if (child != NULL) 
	disconnect( canvas, SIGNAL( sendParam( Point, int, double* ) ),
		    child, SLOT( receiveParam( Point, int, double* ) ) );

  child = new MainWindow();
  child->resize(240,290);
  child->loadPlugin(plugin->ChildName());
  child->setColorMap(canvas->colorMap());

  connect( canvas, SIGNAL( sendParam( Point, int, double* ) ),
	   child, SLOT( receiveParam( Point, int, double* ) ) );
  connect( child, SIGNAL( destroyed() ),
	   this, SLOT( childClosed() ) );
  connect( child, SIGNAL( pluginChanged() ),
	   this, SLOT( disconnectChild() ) );

  child->show();
}

void MainWindow::loadColorMap()
{
  QString fn= QFileDialog::getOpenFileName( QString(COLORMAP_PATH),
					    QString("Colormap (*.map)"),
					    this);
  if ( !fn.isEmpty() ) {
    canvas->loadColorMap(fn);
    draw();
  }
  canvas->repaint(FALSE);
}

void MainWindow::loadPlugin()
{
  QString fn= QFileDialog::getOpenFileName( QString(PLUGIN_PATH),
					    QString("Shared object(*.so)"),
					    this);
  if ( !fn.isEmpty() )
    loadPlugin(fn);
}

void MainWindow::loadPlugin( const char *filename )
{
  plugin= new Plugin(filename);
  if (plugin == NULL) { loadPlugin(); return; }
  setFunc( plugin->Map() );
  setInitFunc( plugin->Init() );
  setIterFunc( plugin->Iter() );
  setParamNum( plugin->ParamNum() );
  for (int i=0; i<paramnum; i++)
    param[i] = plugin->Param(i);
  setMaxIter( plugin->MaxIter() );
  setMaxOrbit( plugin->MaxOrbit() );
  
  setCaption( plugin->Name() );

  if ((plugin->ChildName()==NULL) || (*(plugin->ChildName()) == '\0'))
      actionSetChild->setEnabled( FALSE );
  else
      actionSetChild->setEnabled( TRUE );

  if (child != NULL) 
	disconnect( canvas, SIGNAL( sendParam( Point, int, double* ) ),
		    child, SLOT( receiveParam( Point, int, double* ) ) );

  canvas->setParamNum( paramnum );
  canvas->setParam( param );
  canvas->setPlugin( plugin );
  canvas->setPluginName(filename);

  canvas->draw();

  emit pluginChanged();
}

void MainWindow::orbitChanged()
{
  canvas->setMaxOrbit( maxorbit );
}

void MainWindow::paramChanged()
{
  canvas->setMaxIter( maxiter );

  canvas->setParam( param );
  canvas->draw();
}

void MainWindow::receiveParam( Point z, int m, double *p )
{
  if (paramnum<1) return;
  param[0]=z.x;
  if (paramnum<2) { paramChanged(); return; }
  param[1]=z.y;

  for (int i=0; i<m; i++) {
    if (paramnum <= i+2 ) { paramChanged(); return; }
    param[i+2]=*p++;
  }

  paramChanged();
}

void MainWindow::toggleOrbit()
{
  isOrbit= !isOrbit;
  menuBar()->setItemChecked( SetOrbitId, isOrbit );
  emit orbitChanged( isOrbit );
}

void MainWindow::disconnectChild()
{
  disconnect(canvas, SIGNAL( sendParam( Point, int, double* ) ),
	     child, SLOT( receiveParam( Point, int, double* ) ) );
}
